SUBROUTINE GAUSS2MODEL
!
!**** TITLE - GAUSS2MODEL
!
!     Purpose:
!     --------
!        Transfer an increment field of controlled species from
!        the control Gauss grid model grid to the 
!
!     Interface:
!     ----------
!        Cf. PALM id. card
!
!***  Method:
!     -------
!        For a regular model grid
!          1) including the poles : uses a spherical analysis + synthesis
!             from the gauss control grid to the so called regular grid 
!          2) not including the poles : it acts as in 1 then uses a 
!             Spherepack grid shift from the grid including the poles, 
!             to the so called offset grid.
!        For a gaussian model grid
!          1) if the model and observable grid dimensions differ, uses a 
!             spherical analysis + synthesis from a grid to another
!          2) if the model and observable grid dimensions are the same,
!             is the identity
!
!     Externals:
!     ----------
!        The SpherePack, spherical analysis package. In particular 
!        sshifte: for switching from a regular to an offset grid
!        trssph: for the grid transfer
!
!     Files:
!     ------
!
!     References:
!     -----------
!        Sebastien Massart, technical report on B implementation
!
!     History:
!     --------
!      Version    Programmer      Date         Description
!      -------    ----------      ----         -----------
!        1.0      AP           24/02/2008      Created 
!*--------------------------------------------------------------------------
!
!**   0. DECLARATIONS  
!        ------------
!
!**   0.1 Include files and modules
!
  USE palmlib         !*I The PALM interface
  USE palm_user_param !*I The PrePALM constants
  USE kinds_mod       !*I The SCRIP kind definitions
  USE remap_read      !*I The SCRIP interpolation input
  USE remap_mod       !*I The SCRIP interpolation remapping
!
!**   0.2 Local variables 
!
  IMPLICIT NONE
!
!*V Palm Related Variables
!
  CHARACTER(LEN=PL_LNAME) :: cl_name, cl_space
  INTEGER :: il_time, il_usrtag, il_err, il_err2
  INTEGER :: il_day, il_month, il_year, il_hour, il_min, il_sec
!
!*V Local variables
!
  INTEGER :: il_worktime
  INTEGER :: il_gauss_modelgrid, il_lat_no_poles
!
  INTEGER :: il_off,il_nlon,il_nlat
  INTEGER, SAVE :: il_shlsav, il_shlwork
  REAL, ALLOCATABLE, SAVE, DIMENSION(:) :: rla_shwsav
  REAL, ALLOCATABLE, DIMENSION(:) :: rla_shwork
!
  INTEGER :: il_intl,il_nlona,il_nlata,il_nlonb,il_nlatb
  INTEGER :: il_lsvmin,il_lwkmin
  INTEGER,SAVE :: il_lsave,il_lwork,il_ldwork
!
  REAL, ALLOCATABLE, SAVE, DIMENSION(:) :: rla_wsave
  REAL, ALLOCATABLE, DIMENSION(:) :: rla_work
  REAL(KIND=8), ALLOCATABLE, DIMENSION(:) :: dla_work
!
  INTEGER :: ib_species, ib_lev
!
  LOGICAL, SAVE :: ll_first_time = .TRUE.
  LOGICAL, SAVE :: ll_offset, ll_transfer, ll_gaussmodel
!
  REAL, DIMENSION(ip_lono1,ip_lato1,ip_nivm1,ip_nassim) :: rla_gaussfield
!
  INTEGER, DIMENSION(2) :: ila_grida, ila_gridb
!
  REAL, ALLOCATABLE, TARGET, DIMENSION(:,:) :: rla_greg
!
  REAL, POINTER, DIMENSION(:,:) :: rla_db
!
  REAL, ALLOCATABLE, TARGET, DIMENSION(:,:,:,:) :: rla_modelfield
!
  REAL, ALLOCATABLE, DIMENSION(:) :: rla_fake
  REAL, ALLOCATABLE, DIMENSION(:), SAVE :: rla_fake2
!
!*V SCRIP related routines
!
  CHARACTER(char_len) :: cl_interp_file, cl_map_name
  REAL, DIMENSION(:), ALLOCATABLE :: rla_mod_linarray
  REAL, DIMENSION(:), ALLOCATABLE :: rla_gau_linarray
!
!** 1. Determines the work time and the grid configuration
!
  cl_space = 'one_integer'
  il_time = PL_NO_TIME
  il_usrtag = PL_NO_TAG
  cl_name = 'id_worktime'
  CALL PALM_Get(cl_space, cl_name, il_time, il_usrtag, il_worktime, il_err) 
!
  cl_space = 'one_integer'
  il_time = PL_NO_TIME
  il_usrtag = PL_NO_TAG
  cl_name = 'gauss_modelgrid'
  CALL PALM_Get(cl_space, cl_name, il_time, il_usrtag, il_gauss_modelgrid, &
     il_err) 
!
  cl_space = 'one_integer'
  il_time = PL_NO_TIME
  il_usrtag = PL_NO_TAG
  cl_name = 'lat_no_poles'
  CALL PALM_Get(cl_space, cl_name, il_time, il_usrtag, il_lat_no_poles, &
     il_err) 
!
  IF (ip_iprint .gt. 0) THEN
      WRITE(PL_OUT,*) 'Working for time ',il_worktime
      IF (il_gauss_modelgrid .EQ. 0) THEN
	  IF (il_lat_no_poles .EQ. 0) THEN
	      WRITE(PL_OUT,*) 'Working for a Regular grid'
	  ELSE
	      WRITE(PL_OUT,*) 'Working for an offset grid'
	  ENDIF
      ELSE
	  WRITE(PL_OUT,*) 'Working for a Gaussian grid'
      ENDIF
  ENDIF
!
!** 2. Select the tasks to perform
!
  IF (ll_first_time) THEN
      ll_transfer = ((il_gauss_modelgrid.EQ.0).OR.&
	 ((il_gauss_modelgrid.EQ.1).AND.&
	 ((ip_lonm1.NE.ip_lono1).OR.(ip_latm1.NE.ip_lato1))))
      ll_gaussmodel = (il_gauss_modelgrid.EQ.1)
      ll_offset = ((il_gauss_modelgrid.EQ.0).AND.(il_lat_no_poles.EQ.1))
!
      IF (ip_iprint .gt. 0) THEN
	  WRITE(PL_OUT,*) 'Performing a grid transfert ',ll_transfer
	  WRITE(PL_OUT,*) 'Shifting an offset to a regular grid ',ll_offset
      ENDIF
  ENDIF
!
!** 3. Get the input fields
!
  cl_space = 'gauss_space'
  il_time = il_worktime
  il_usrtag = PL_NO_TAG
  cl_name = 'rda_3D_gau'
  CALL PALM_Get(cl_space, cl_name, il_time, il_usrtag, rla_gaussfield, &
     il_err)
!
!** 4. Decide wether to apply a transform and wich one
!
  IF (ll_transfer) THEN
!
!** 5. Set the grid dimensions and switches
!
      IF (ll_offset) THEN
	  il_nlona = ip_lono1
	  il_nlata = ip_lato1
	  il_nlonb = ip_lonm1
	  il_nlatb = ip_latm1+1
	  il_nlon = ip_lonm1
	  il_nlat = ip_latm1
      ELSE
	  il_nlona = ip_lono1
	  il_nlata = ip_lato1
	  il_nlonb = ip_lonm1
	  il_nlatb = ip_latm1
      ENDIF
!
      ila_grida = (/2,0/)
      ila_gridb = (/1,0/)
      IF (ll_gaussmodel) ila_gridb = (/2,0/)
!
!** 6. Allocate the output field
!
      ALLOCATE(rla_modelfield(ip_lonm1,ip_latm1,ip_nivm1,ip_nassim))
      rla_modelfield(:,:,:,:)=0.
!
!** 7. Determine the workspace sizes and allocate static workspaces
!
      IF (ll_first_time) THEN
	  IF (ip_scrip .eq. 0) THEN
	      il_intl=0
	      il_lsave=0
	      il_lwork=0
	      il_ldwork=0
	      CALL trssph(il_intl,&
		 ila_grida,il_nlona,il_nlata,rla_gaussfield(:,:,1,1),&
		 ila_gridb,il_nlonb,il_nlatb,rla_modelfield(:,:,1,1),&
		 rla_wsave,il_lsave,il_lsvmin,&
		 rla_work,il_lwork,il_lwkmin,&
		 dla_work,il_ldwork,il_err)
!
	      il_lsave=il_lsvmin
	      il_lwork=il_lwkmin
	      il_ldwork=MAX(il_nlata,il_nlatb)*(MAX(il_nlata,il_nlatb)+4)
!
	      ALLOCATE(rla_wsave(il_lsave))
	      rla_wsave=0.
!
	      IF (ll_offset) THEN
		  il_shlsav = 2*(2*il_nlat+il_nlon+16)
		  IF (MOD(il_nlon,2).eq.0) THEN
		      il_shlwork = 2*il_nlon*(il_nlat+1)
		  ELSE
		      il_shlwork = il_nlon*(5*il_nlat+1)
		  ENDIF
		  ALLOCATE(rla_shwsav(il_shlsav))
		  rla_shwsav(:)=0.
		  il_off = 1
		  CALL sshifti(il_off,il_nlon,il_nlat,il_shlsav,&
		     rla_shwsav,il_err)
		  IF (il_err .ne. 0) THEN
		      WRITE(PL_OUT,*) 'Error in gauss2model sshifti call'
		      WRITE(*,*) 'Error in gauss2model sshifti call'
		      CALL PALM_Abort(il_err) 
		  ENDIF
	      ENDIF
	  ELSE
!
!** SCRIP A : load map weights
!
	      cl_interp_file='rmp_control_to_model_bilin.nc'
	      CALL read_remap2(cl_map_name, cl_interp_file)
	  ENDIF
!
      ENDIF
!
!** 8. Allocate volatile workspaces
!
      IF (ip_scrip .eq. 0) THEN
	  ALLOCATE(rla_work(il_lwork))
	  rla_work(:)=0.
	  ALLOCATE(dla_work(il_ldwork))
	  dla_work(:)=0.0
	  IF (ll_offset) THEN
	      ALLOCATE(rla_shwork(il_shlwork))
	      rla_shwork(:)=0.
	      ALLOCATE(rla_greg(ip_lonm1,ip_latm1+1))
	      rla_greg(:,:)=0.
	  ENDIF
      ELSE
	  ALLOCATE (rla_mod_linarray(ip_latm1*ip_lonm1))
	  ALLOCATE (rla_gau_linarray(ip_lato1*ip_lono1))
      ENDIF
!
!** 9. Loop over the transforms
!
      il_intl = 0
      DO ib_species = 1, ip_nassim
	DO ib_lev = 1, ip_nivm1
	  IF (ip_scrip .eq. 0) THEN
	      IF (ll_offset) THEN
		  rla_db => rla_greg 
	      ELSE
		  rla_db => rla_modelfield(:,:,ib_lev,ib_species)
	      ENDIF
	      CALL trssph(il_intl,&
		 ila_grida,il_nlona,il_nlata,&
		 rla_gaussfield(:,:,ib_lev,ib_species),&
		 ila_gridb,il_nlonb,il_nlatb,&
		 rla_db,&
		 rla_wsave,il_lsave,il_lsvmin,&
		 rla_work,il_lwork,il_lwkmin,&
		 dla_work,il_ldwork,il_err)
	      IF (il_err .ne. 0) THEN
		  WRITE(PL_OUT,*) 'Error in gauss2model trssph call'
		  WRITE(*,*) 'Error in gauss2model tressph call'
		  CALL PALM_Abort(il_err) 
	      ENDIF
	      il_intl=1
	      IF (ll_offset) THEN
		  il_off=1
		  CALL sshifte(il_off,il_nlon,il_nlat,&
		     rla_modelfield(:,:,ib_lev,ib_species),rla_greg,&
		     rla_shwsav,il_shlsav,rla_shwork,il_shlwork,il_err)
		  IF (il_err .ne. 0) THEN
		      WRITE(PL_OUT,*) 'Error in gauss2model sshifte call'
		      WRITE(*,*) 'Error in gauss2model sshifte call'
		      CALL PALM_Abort(il_err) 
		  ENDIF
	      ENDIF
!
!** SCRIP B : level per level remap
!
	  ELSE
	      rla_gau_linarray(:) = RESHAPE(&
		 rla_gaussfield(:,:,ib_lev,ib_species),(/ip_lono1*ip_lato1/))
	      CALL remap(rla_mod_linarray,wts_map2, grid2_add_map2, &
		 grid1_add_map2,rla_gau_linarray)
	      rla_modelfield(:,:,ib_lev,ib_species) = RESHAPE(&
		 rla_mod_linarray(:),(/ip_lonm1,ip_latm1/))
	  ENDIF
!
	END DO
      END DO
!
!** x. Send the results
!
      cl_space = 'model_space'
      il_time = il_worktime
      il_usrtag = PL_NO_TAG
      cl_name = 'rda_3D_mod'
      CALL PALM_Put(cl_space, cl_name, il_time, il_usrtag, rla_modelfield, &
	 il_err)      
      IF (il_err.ne.0) CALL PALM_Error_explain(il_err, il_err2)
!
!** x. Free up local space
!      
      DEALLOCATE(rla_modelfield)
      IF (ip_scrip .eq. 0) THEN
	  DEALLOCATE(rla_work)
	  DEALLOCATE(dla_work)
	  IF (ll_offset) THEN
	      DEALLOCATE(rla_shwork)
	      DEALLOCATE(rla_greg)
	  ENDIF
      ELSE
	  DEALLOCATE(rla_mod_linarray)
	  DEALLOCATE(rla_gau_linarray)
      ENDIF	  
!
  ELSE
!
!** ZZ If no transform is needed, just make the input object pass by
!
      cl_space = 'model_space'
      il_time = il_worktime
      il_usrtag = PL_NO_TAG
      cl_name = 'rda_3D_mod'
      CALL PALM_Put(cl_space, cl_name, il_time, il_usrtag, rla_gaussfield, &
	 il_err)
      IF (il_err.ne.0) CALL PALM_Error_explain(il_err, il_err2)
!      
  ENDIF
!
!** NN. Ends up the unit
!
  IF (ll_first_time) ll_first_time = .FALSE.
  IF (ip_iprint .gt. 0) THEN
      WRITE(*,*) 'Ending transform gauss2model'
      WRITE(PL_OUT,*) 'Ending transform gauss2model'
      CALL FLUSH(PL_OUT)
  ENDIF
!
END SUBROUTINE GAUSS2MODEL
