SUBROUTINE INISTATS
!
!**** TITLE - INISTATS
!
!     Purpose:
!     --------
!        Initializes the background error statistics (variance and
!        correlations).
!
!     Interface:
!     ----------
!        Via PALM
!
!***  Method:
!     -------
!        For the variance:
!         1) for the 1 mode, it receives the controlled species and
!            compute the std err as a % (ip_sigbck) of the controlled
!            species mixing ratio
!         2) for the 2 mode, the variance is read from a statistics file.
!            It takes the form of a 1D vector (size ip_nivm1) containing
!            the variance for each level
!         [3) for the 3 mode, the full 3D variance is read from a 
!            statistics file] NOT IMPLEMENTED YET
!        For the horizontal correlation:
!         1) for the 1 mode, the correlation length is constant everywhere
!            and estimated with the ip_flsh parameter
!         2) for the 2 mode, the correlation length is read from a statistics 
!            file. It takes the form of a 1D vector (size ip_nivm1) containing
!            the hor. correlation length for each level
!         [3) for the 3 mode, the full 3D hor. correlation is read from a 
!            statistics file] NOT IMPLEMENTED YET
!        For the vertical correlation:
!         1) for the 1 mode, the correlation length is constant everywhere
!            and estimated with the ip_flsh parameter
!         2) for the 2 mode, the correlation length is read from a statistics 
!            file. It takes the form of a 1D vector (size ip_nivm1) containing
!            the anysotropic vert. correlation length along a profile. The
!            profile does not vary from a point to another
!         [3) for the 3 mode, the full 3D vert. correlation is read from a 
!            statistics file] NOT IMPLEMENTED YET
!   
!
!     Externals:
!     ----------
!
!     Files:
!     ------
!         The NetCDF diagnosed statistics
!
!     References:
!     -----------
!
!     History:
!     --------
!      Version    Programmer      Date         Description
!      -------    ----------      ----         -----------
!        1.0         AP        29/02/2008      Created
!
!*--------------------------------------------------------------------------
!
!**   0. DECLARATIONS  
!        ------------
!
!**   0.1 Include files and modules
!
  USE palmlib          !*I The PALM interface
  USE palm_user_param  !*I The PrePALM constants
  USE netcdf           !*I The NetCDF I/O library
!
!**   0.2 Local variables 
!
  IMPLICIT NONE
!
!*V Palm Related Variables
!
  CHARACTER(LEN=PL_LNAME) :: cl_name, cl_space
  INTEGER :: il_time, il_usrtag, il_err, il_err2
  INTEGER :: il_day, il_month, il_year, il_hour, il_min, il_sec
!
!*V Local variables
  INTEGER :: ib, ib_spe
  INTEGER :: il_assimtime
  INTEGER :: il_var_kind, il_sigbck
  INTEGER :: il_hcor_kind, il_flsh
  INTEGER :: il_vcor_kind, il_flsz
  REAL,DIMENSION(ip_lonm1,ip_latm1,ip_nivm1,ip_nassim) :: rla_sigmab
  REAL,DIMENSION(ip_nivm1) :: rla_lph, rla_lpz
  REAL,DIMENSION(:),ALLOCATABLE :: rla_varprofile
!
!*V NetCDF related variables
  INTEGER, DIMENSION(5) :: ila_date0
  LOGICAL, SAVE :: ll_firsttime = .true.
  CHARACTER(LEN=16) , ALLOCATABLE , SAVE  :: cla_assim_spe(:)
  CHARACTER(LEN=28) :: cd_file
  CHARACTER(LEN=28) :: cl_seek
  INTEGER :: il_status, il_ncid,il_varid
  REAL(KIND=4),DIMENSION(ip_nivm1) :: rla_read_profile
  REAL(KIND=4),DIMENSION(:,:,:),ALLOCATABLE :: rla_read_3d
! 
!**   1. Get the execution switches
!
!* 1.1 Working time
!
  cl_space = 'one_integer'
  il_time = PL_NO_TIME
  il_usrtag = PL_NO_TAG
  cl_name = 'id_assimtime'
  CALL PALM_Get(cl_space, cl_name, il_time, il_usrtag, il_assimtime, il_err) 
  IF (il_err.ne.0) THEN
      CALL PALM_Error_explain(il_err, il_err2)
      CALL PALM_Abort(il_err) 
  ENDIF
!
!* 1.2 Variance representation
!
  cl_space = 'one_integer'
  il_time = PL_NO_TIME
  il_usrtag = PL_NO_TAG
  cl_name = 'variance_representation'
  CALL PALM_Get(cl_space, cl_name, il_time, il_usrtag, il_var_kind, il_err) 
  IF (il_err.ne.0) THEN
      CALL PALM_Error_explain(il_err, il_err2)
      CALL PALM_Abort(il_err) 
  ENDIF
!
!* 1.3 Hor. correl. representation
!
  cl_space = 'one_integer'
  il_time = PL_NO_TIME
  il_usrtag = PL_NO_TAG
  cl_name = 'h_correl_representation'
  CALL PALM_Get(cl_space, cl_name, il_time, il_usrtag, il_hcor_kind, il_err) 
  IF (il_err.ne.0) THEN
      CALL PALM_Error_explain(il_err, il_err2)
      CALL PALM_Abort(il_err) 
  ENDIF
!
!* 1.4 Vert. correl. representation
!
  cl_space = 'one_integer'
  il_time = PL_NO_TIME
  il_usrtag = PL_NO_TAG
  cl_name = 'v_correl_representation'
  CALL PALM_Get(cl_space, cl_name, il_time, il_usrtag, il_vcor_kind, il_err) 
  IF (il_err.ne.0) THEN
      CALL PALM_Error_explain(il_err, il_err2)
      CALL PALM_Abort(il_err) 
  ENDIF
!
!* 1.5 Open the NetCDF background error statistics file if needed
!
  IF (il_var_kind.GT.1.OR.il_hcor_kind.GT.1.OR.il_vcor_kind.GT.1) THEN
!
      cl_space = 'date'
      il_time = PL_NO_TIME
      il_usrtag = PL_NO_TAG
      cl_name = 'ida_date0'
      CALL PALM_Get(cl_space,cl_name,il_time,il_usrtag,ila_date0,il_err) 
!
      WRITE(cd_file,fmt='(''BKG_ERR_STATS+'',I4.4,I2.2,''.nc'')')&
	 ila_date0(1),ila_date0(2)
!
      IF (ll_firsttime) THEN
	  ALLOCATE( cla_assim_spe(ip_nassim) )
!
	  cl_space = 'assim_names'
	  il_time = PL_NO_TIME
	  il_usrtag = PL_NO_TAG
	  cl_name = 'assim_species'
	  CALL PALM_Get(cl_space,cl_name,il_time,il_usrtag,cla_assim_spe,&
	     il_err)	
!
	  ll_firsttime = .false.
      ENDIF
!
      il_status=NF90_OPEN(TRIM(cd_file),NF90_NOWRITE,il_ncid)
      IF (il_STATUS .ne. NF90_NOERR) THEN
          WRITE(*,*) 'Error opening ',trim(cd_file)
          WRITE(PL_OUT,*) 'Error opening ',trim(cd_file)
          CALL PALM_Abort(il_err)
      ENDIF
  ENDIF
!
!--------------------------------------------
!
!** 2. Compute the variance
!
!* Representation switch
!
  SELECT CASE(il_var_kind)
  CASE(1)
!
!* 2.1 Constant from percent std err
!
      WRITE(*,'(A44)') &
      '++ INITIALIZING CONSTANT BACKGROUND VARIANCE'
      WRITE(PL_OUT,'(A44)') &
      '++ INITIALIZING CONSTANT BACKGROUND VARIANCE'
!
!* 2.1.1 Get the percent std err
!
      cl_space = 'one_integer'
      il_time = PL_NO_TIME
      il_usrtag = PL_NO_TAG
      cl_name = 'id_sigbck'
      CALL PALM_Get(cl_space, cl_name, il_time, il_usrtag, il_sigbck, il_err) 
      IF (il_err.ne.0) THEN
	  CALL PALM_Error_explain(il_err, il_err2)
	  CALL PALM_Abort(il_err) 
      ENDIF
!
!* 2.1.2 Get the model species concentration
!
      cl_space = 'controlled_species'
      il_time = il_assimtime
      il_usrtag = PL_NO_TAG
      cl_name = 'rda_3D_mod'
      CALL PALM_Get(cl_space, cl_name, il_time, il_usrtag, rla_sigmab, il_err) 
      IF (il_err.ne.0) THEN
	  CALL PALM_Error_explain(il_err, il_err2)
	  CALL PALM_Abort(il_err) 
      ENDIF
!
!* 2.1.3 Diagnostic output
!
      WRITE(*,'(A24,A14)') ' + COMPUTING BACKGROUND ',&
	 'ERROR VARIANCE'
      WRITE(*,'(A26,I3,A1)')  &
	 ' + USING A % STD. ERR. OF ', il_sigbck,'%'
      WRITE(PL_OUT,'(A24,A14)') ' + COMPUTING BACKGROUND ', &
	 'ERROR VARIANCE'
      WRITE(PL_OUT,'(A26,I3,A1)') &
         ' + USING A % STD. ERR. OF ', il_sigbck,'%'
!
!* 2.1.4 Compute the variance
!
	rla_sigmab(:,:,:,:) = &
	   rla_sigmab(:,:,:,:) * REAL(il_sigbck)/100.
!
  CASE (2)
!
!* 2.2 Constant per level from 1D read profile
!
      WRITE(*,'(A72)') &
      '++ INITIALIZING CONSTANT PER LEVEL BACKGROUND VARIANCE FROM A 1D PROFILE'
      WRITE(*,'(A13,A28)') ' + FROM FILE ',trim(cd_file)
      WRITE(PL_OUT,'(A72)') &
      '++ INITIALIZING CONSTANT PER LEVEL BACKGROUND VARIANCE FROM A 1D PROFILE'      
      WRITE(PL_OUT,'(A13,A28)') ' + FROM FILE ',TRIM(cd_file)
!
!* 2.2.1 Allocation
!
      ALLOCATE(rla_varprofile(ip_nivm1),stat=il_err)
      IF (il_err.ne.0) THEN
	  WRITE(*,*) 'Error allocating rla_varprofile'
	  WRITE(*,*) 'Shutting down the simulation'
	  WRITE(PL_OUT,*) 'Error allocating rla_varprofile'
	  WRITE(PL_OUT,*) 'Shutting down the simulation'
	  CALL PALM_Abort(il_err) 
      ENDIF
!
!* 2.2.2 Load the variance profile
!
      DO ib_spe = 1, ip_nassim
	cl_seek=trim(cla_assim_spe(ib_spe))//"_var_profile"
	il_status=NF90_INQ_VARID(il_ncid,TRIM(cl_seek),il_varid)
	IF (il_status .eq. NF90_NOERR) THEN
	    il_status=NF90_GET_VAR(il_ncid,il_varid,rla_read_profile)
	    rla_varprofile(:)=DBLE(rla_read_profile(:))
	ELSE
	    WRITE(*,*) 'Variable ',TRIM(cl_seek),&
	       ' not in FILE ',TRIM(cd_file)
	    WRITE(PL_OUT,*) 'Variable ',TRIM(cl_seek),&
	       ' not in file ',TRIM(cd_file)
	    CALL PALM_Abort(il_err)
	ENDIF
!
!* 2.2.3 Compute the variance
!
	DO ib=1, ip_nivm1
	  rla_sigmab(:,:,ib,ib_spe) = rla_varprofile(ib)
	ENDDO
      END DO
!
!* 2.2.4 Free memory
!
      DEALLOCATE(rla_varprofile)
!      
  CASE(3)
!
!* 2.3 Full 3D variance
!
      WRITE(*,'(A43)') &
      '++ INITIALIZING FULL 3D BACKGROUND VARIANCE'
      WRITE(*,'(A13,A28)') ' + FROM FILE ',trim(cd_file)
      WRITE(PL_OUT,'(A72)') &
      '++ INITIALIZING FULL 3D BACKGROUND VARIANCE'      
      WRITE(PL_OUT,'(A13,A28)') ' + FROM FILE ',TRIM(cd_file)
!
!* 2.3.1 Allocation
!
      ALLOCATE(rla_read_3d(ip_lonm1,ip_latm1,ip_nivm1),stat=il_err)
      IF (il_err.ne.0) THEN
	  WRITE(*,*) 'Error allocating rla_read_3d'
	  WRITE(*,*) 'Shutting down the simulation'
	  WRITE(PL_OUT,*) 'Error allocating rla_read_3d'
	  WRITE(PL_OUT,*) 'Shutting down the simulation'
	  CALL PALM_Abort(il_err) 
      ENDIF
!
!* 2.3.2 Load the variance 
!
      DO ib_spe = 1, ip_nassim
	cl_seek=TRIM(cla_assim_spe(ib_spe))//"_var_3d"
	il_status=NF90_INQ_VARID(il_ncid,TRIM(cl_seek),il_varid)
	IF (il_status .eq. NF90_NOERR) THEN
	    il_status=NF90_GET_VAR(il_ncid,il_varid,rla_read_3d)
	    rla_sigmab(:,:,:,ib_spe)=DBLE(rla_read_3d(:,:,:))
	ELSE
	    WRITE(*,*) 'Variable ',TRIM(cl_seek),&
	       ' not in FILE ',TRIM(cd_file)
	    WRITE(PL_OUT,*) 'Variable ',TRIM(cl_seek),&
	       ' not in file ',TRIM(cd_file)
	    CALL PALM_Abort(il_err)
	ENDIF
      END DO
!
!* 2.2.4 Free memory
!
      DEALLOCATE(rla_read_3d)
!
  CASE DEFAULT
!
!* 2.4 Stop in case of wrong switch
!
      WRITE(*,*) 'Variance representation switch out of range'
      WRITE(*,*) 'Shutting down the simulation'
      WRITE(PL_OUT,*) 'Variance representation switch out of range'
      WRITE(PL_OUT,*) 'Shutting down the simulation'
      CALL PALM_Abort(il_err)
  END SELECT
!
!* 2.5 Put of the Variance estimate
!
  cl_space = 'controlled_species'
  il_time = il_assimtime
  il_usrtag = PL_NO_TAG
  cl_name = 'rda_sigmab'
  CALL PALM_Put(cl_space, cl_name, il_time, il_usrtag, rla_sigmab, il_err) 
  IF (il_err.ne.0) THEN
      CALL PALM_Error_explain(il_err, il_err2)
      CALL PALM_Abort(il_err) 
  ENDIF
!
!--------------------------------------------
!
!** 3. Compute the horizontal correlation
!
!--- Temporary cut off
!
  IF (.FALSE.) THEN
!
!* Representation switch
!
  SELECT CASE(il_hcor_kind)
  CASE(1)
!
!* 2.1 Fixed horizontal correl. length
!
      WRITE(*,'(A2)') '++'
      WRITE(*,'(A44)') &
      '++ INITIALIZING FIXED HORIZONTAL CORRELATION'
      WRITE(PL_OUT,'(A44)') &
      '++ INITIALIZING FIXED HORIZONTAL CORRELATION'
!
!* 2.1.1 Get the scale factor
!
      cl_space = 'one_integer'
      il_time = PL_NO_TIME
      il_usrtag = PL_NO_TAG
      cl_name = 'filter_h'
      CALL PALM_Get(cl_space, cl_name, il_time, il_usrtag, il_flsh, il_err) 
      IF (il_err.ne.0) THEN
	  CALL PALM_Error_explain(il_err, il_err2)
	  CALL PALM_Abort(il_err) 
      ENDIF
!
      IF (il_flsh .NE. 0) THEN
	  WRITE(*,'(A24,A30)') '++ HORIZONTAL BKG. ERR. ', &
	     'CORRELATION TAKEN INTO ACCOUNT'
	  WRITE(*,'(A25,I3)') '++ LENGTH SCALE (deg.) = ', il_flsh
	  WRITE(PL_OUT,'(A24,A30)') '++ HORIZONTAL BKG. ERR. ', &
	     'CORRELATION TAKEN INTO ACCOUNT'
	  WRITE(PL_OUT,'(A25,I3)') '++ LENGTH SCALE (deg.) = ', il_flsh
      ENDIF
!
  CASE DEFAULT
!
!* 2.4 Stop in case of wrong switch
!
      WRITE(*,*) 'Hor correl representation switch out of range'
      WRITE(*,*) 'Shutting down the simulation'
      WRITE(PL_OUT,*) 'Hor correl representation switch out of range'
      WRITE(PL_OUT,*) 'Shutting down the simulation'
      CALL PALM_Abort(il_err)
  END SELECT
!
!* 2.5 Horizontal length scale profile
!
  cl_space = 'vert_vector'
  il_time = il_assimtime
  il_usrtag = PL_NO_TAG
  cl_name = 'rda_lph'
  CALL PALM_Put(cl_space, cl_name, il_time, il_usrtag, rla_lph, il_err) 
  IF (il_err.ne.0) THEN
      CALL PALM_Error_explain(il_err, il_err2)
      CALL PALM_Abort(il_err) 
  ENDIF
!
  ENDIF
!
!--- Temporary cut off end
!
!--------------------------------------------
!
!** 3. Compute the vertical correlation
!
!* Representation switch
!
  SELECT CASE(il_vcor_kind)
  CASE(1)
!
!* 3.1 Fixed vertical correl. length from il_flsz
!
      WRITE(*,'(A2)') '++'
      WRITE(*,'(A42)') &
      '++ INITIALIZING FIXED VERTICAL CORRELATION'
      WRITE(PL_OUT,'(A42)') &
      '++ INITIALIZING FIXED VERTICAL CORRELATION'
!
!* 3.1.1 Get the exponential scale factor
!
      cl_space = 'one_integer'
      il_time = PL_NO_TIME
      il_usrtag = PL_NO_TAG
      cl_name = 'filter_z'
      CALL PALM_Get(cl_space, cl_name, il_time, il_usrtag, il_flsz, il_err) 
      IF (il_err.ne.0) THEN
	  CALL PALM_Error_explain(il_err, il_err2)
	  CALL PALM_Abort(il_err) 
      ENDIF
!
      IF (il_flsz .NE. 0) THEN
	  WRITE(*,'(A22,A30)') '++ VERTICAL BKG. ERR. ',&
	     'CORRELATION TAKEN INTO ACCOUNT'
	  WRITE(*,'(A25,I3)') '++ log(p) LENGTH SCALE = ', il_flsz
	  WRITE(PL_OUT,'(A22,A30)') '++ VERTICAL BKG. ERR. ', &
	     'CORRELATION TAKEN INTO ACCOUNT'
	  WRITE(PL_OUT,'(A25,I3)') '++ log(p) LENGTH SCALE = ', il_flsz
!
!* 3.1.2 Compute the length scale
!
	  rla_lpz(:) = 1./SQRT(2.*REAL(il_flsz))
      ELSE
	  rla_lpz(:) = 0.
      ENDIF
      WRITE(*,'(A2)') '++'
      WRITE(PL_OUT,'(A2)') '++'
!
  CASE (2)
!
!* 2.2 Anisotropic 1D profile, constant per gridpoint
!
      WRITE(*,'(A2)') '++'
      WRITE(*,'(A71)') &
      '++ INITIALIZING ANISOTROPIC VERTICAL CORRELATION CONSTANT PER GRIDPOINT'
      WRITE(*,'(A13,A28)') ' + FROM FILE ',trim(cd_file)
      WRITE(*,'(A2)') '++'
      WRITE(PL_OUT,'(A71)') &
      '++ INITIALIZING ANISOTROPIC VERTICAL CORRELATION CONSTANT PER GRIDPOINT'
      WRITE(PL_OUT,'(A13,A28)') ' + FROM FILE ',trim(cd_file)
!
      il_flsz = 1
!
!* 2.2.2 Load the correlation profile
!
      DO ib_spe = 1, ip_nassim
	cl_seek=trim(cla_assim_spe(ib_spe))//"_v_cor_profile"
	il_status=NF90_INQ_VARID(il_ncid,TRIM(cl_seek),il_varid)
	IF (il_status .eq. NF90_NOERR) THEN
	    il_status=NF90_GET_VAR(il_ncid,il_varid,rla_read_profile)
	    rla_lpz(:)=DBLE(rla_read_profile(:))
	ELSE
	    WRITE(*,*) 'Variable ',TRIM(cl_seek),&
	       ' not in FILE ',TRIM(cd_file)
	    WRITE(PL_OUT,*) 'Variable ',TRIM(cl_seek),&
	       ' not in file ',TRIM(cd_file)
	    CALL PALM_Abort(il_err)
	ENDIF
      END DO
!
  CASE(3)
!
!* 4.3 Full 3D variance
!
      WRITE(*,*) 'Vert. corr. representation switch 3 not yet implemented'
      WRITE(*,*) 'Shutting down the simulation'
      WRITE(PL_OUT,*) 'Vert. corr. representation switch 3 not yet implemented'
      WRITE(PL_OUT,*) 'Shutting down the simulation'
      CALL PALM_Abort(il_err)
!
  CASE DEFAULT
!
!* 4.4 Stop in case of wrong switch
!
      WRITE(*,*) 'Verr correl representation switch out of range'
      WRITE(*,*) 'Shutting down the simulation'
      WRITE(PL_OUT,*) 'Ver correl representation switch out of range'
      WRITE(PL_OUT,*) 'Shutting down the simulation'
      CALL PALM_Abort(il_err)
  END SELECT
!
!* 4.5 Put of the vertical length scale profile
!
  IF (il_flsz .NE. 0) THEN
      cl_space = 'vert_vector'
      il_time = il_assimtime
      il_usrtag = PL_NO_TAG
      cl_name = 'rda_lpz'
      CALL PALM_Put(cl_space, cl_name, il_time, il_usrtag, rla_lpz, il_err) 
      IF (il_err.ne.0) THEN
	  CALL PALM_Error_explain(il_err, il_err2)
	  CALL PALM_Abort(il_err) 
      ENDIF
  ENDIF
!
!** 5. Close the NetCDF Background error statistics file
!
  IF (il_var_kind.GT.1.OR.il_hcor_kind.GT.1.OR.il_vcor_kind.GT.1) THEN
      il_status=NF90_CLOSE(il_ncid)
!      DEALLOCATE(cla_assim_spe)
  ENDIF
!
!=======================================================================
!
END SUBROUTINE INISTATS

